import pygame

from OpenGL.GL import *
from OpenGL.GLU import *

class Screen:
  def __init__(self):
    self.wi = 640
    self.hi = 480
    self.fullscreen = 0
    self.caption = "Nehe lesson 18 - quadrics"
    self.glinited = False
    self.texloaded = False
    self.tnames = []
    self.textures = []
    
    self.lighting = False
    self.ambientlight = (.5,.5,.5,1)  #halfbright
    self.diffuselight = (1,1,1,1)  #fullbright
    self.lightpos = (0,0,2,1)   #a little in front of screen
    
    #instance variables to be replaced with objects
    self.xrot = 0
    self.yrot = 0
    self.xspeed = 0
    self.yspeed = 0
    self.z = -5
    self.filter = 0 #which texture to use
    
    self.discstart = 0
    self.discend = 0
    self.inc1 = 0
    self.inc2 = 1
    self.quadratic = None
    self.whichobject = 0
  def initGL(self):
    if not self.texloaded: self.loadGLTextures()
    glEnable(GL_TEXTURE_2D)
    glShadeModel(GL_SMOOTH)
    glClearColor(0,0,0,0)
    glClearDepth(1.)
    glEnable(GL_DEPTH_TEST)
    glDepthFunc(GL_LEQUAL)
    glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST)
    glLightfv(GL_LIGHT1,GL_AMBIENT,self.ambientlight)
    glLightfv(GL_LIGHT1,GL_DIFFUSE,self.diffuselight)
    glLightfv(GL_LIGHT1,GL_POSITION,self.lightpos)
    glEnable(GL_LIGHT1)
    
    #init quadratic
    self.quadratic = gluNewQuadric()
    gluQuadricNormals(self.quadratic,GLU_SMOOTH)
    gluQuadricTexture(self.quadratic,GL_TRUE)
    #self.glinited = True
  def loadGLTextures(self):
    self.textures.append(pygame.image.load("Data/Crate.bmp"))
    idat = pygame.image.tostring(self.textures[0],"RGB",True)
    self.tnames = glGenTextures(3)
    if not hasattr(self.tnames,"__len__"):
      self.tnames = (self.tnames,)
    #near filter
    glBindTexture(GL_TEXTURE_2D,self.tnames[0])
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST)
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST)
    glTexImage2D(GL_TEXTURE_2D,0,3,self.textures[0].get_width(),self.textures[0].get_height(),0,GL_RGB,GL_UNSIGNED_BYTE,idat)
    #linear filter
    glBindTexture(GL_TEXTURE_2D,self.tnames[1])
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR)
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR)
    glTexImage2D(GL_TEXTURE_2D,0,3,self.textures[0].get_width(),self.textures[0].get_height(),0,GL_RGB,GL_UNSIGNED_BYTE,idat)
    #mipmapped texture
    glBindTexture(GL_TEXTURE_2D,self.tnames[2])
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST)
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR)
    gluBuild2DMipmaps(GL_TEXTURE_2D,3,self.textures[0].get_width(),self.textures[0].get_height(),GL_RGB,GL_UNSIGNED_BYTE,idat)
    #glTexImage2D(GL_TEXTURE_2D,0,3,self.textures[0].get_width(),self.textures[0].get_height(),0,GL_RGB,GL_UNSIGNED_BYTE,idat)
    #self.textures[0] = None
    #self.texloaded = True
    self.textures = []
  def resize(self):
    wi,hi,fullscreen = self.wi,self.hi,self.fullscreen
    self.screen = pygame.display.set_mode([wi,hi],pygame.OPENGL|pygame.DOUBLEBUF|pygame.FULLSCREEN*fullscreen|pygame.RESIZABLE)
    pygame.display.set_caption(self.caption)
    if hi==0: hi = 1
    glViewport(0,0,wi,hi)
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    gluPerspective(45.,float(wi)/float(hi),.1,100.)
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()
    if not self.glinited: self.initGL()
  def drawcube(self):
    glBegin(GL_QUADS)
    #front
    glNormal3f(0,0,1)
    glTexCoord2f(0,0);glVertex3f(-1,-1,1)
    glTexCoord2f(1,0);glVertex3f(1,-1,1)
    glTexCoord2f(1,1);glVertex3f(1,1,1)
    glTexCoord2f(0,1);glVertex3f(-1,1,1)
    #back
    glNormal3f(0,0,-1)
    glTexCoord2f(0,0);glVertex3f(1,-1,-1)
    glTexCoord2f(1,0);glVertex3f(-1,-1,-1)
    glTexCoord2f(1,1);glVertex3f(-1,1,-1)
    glTexCoord2f(0,1);glVertex3f(1,1,-1)
    #left
    glNormal3f(-1,0,0)
    glTexCoord2f(0,0);glVertex3f(-1,-1,-1)
    glTexCoord2f(1,0);glVertex3f(-1,-1,1)
    glTexCoord2f(1,1);glVertex3f(-1,1,1)
    glTexCoord2f(0,1);glVertex3f(-1,1,-1)
    #right
    glNormal3f(1,0,0)
    glTexCoord2f(0,0);glVertex3f(1,-1,1)
    glTexCoord2f(1,0);glVertex3f(1,-1,-1)
    glTexCoord2f(1,1);glVertex3f(1,1,-1)
    glTexCoord2f(0,1);glVertex3f(1,1,1)
    #top
    glNormal3f(0,1,0)
    glTexCoord2f(0,0);glVertex3f(-1,1,1)
    glTexCoord2f(1,0);glVertex3f(1,1,1)
    glTexCoord2f(1,1);glVertex3f(1,1,-1)
    glTexCoord2f(0,1);glVertex3f(-1,1,-1)
    #bottom
    glNormal3f(0,-1,0)
    glTexCoord2f(0,0);glVertex3f(-1,-1,-1)
    glTexCoord2f(1,0);glVertex3f(1,-1,-1)
    glTexCoord2f(1,1);glVertex3f(1,-1,1)
    glTexCoord2f(0,1);glVertex3f(-1,-1,1)
    glEnd()
  def draw(self):
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT)
    glLoadIdentity()
    glTranslatef(0,0,self.z)
    glRotatef(self.xrot,1,0,0)
    glRotatef(self.yrot,0,1,0)
    #select texture
    glBindTexture(GL_TEXTURE_2D,self.tnames[self.filter])
    if self.whichobject == 0:
      self.drawcube()
    elif self.whichobject == 1:
      glTranslatef(0,0,-1.5)
      gluCylinder(self.quadratic,1,1,3,32,32)
    elif self.whichobject == 2:
      gluDisk(self.quadratic,.5,1.5,32,32)
    elif self.whichobject == 3:
      gluSphere(self.quadratic,1.3,32,32)
    elif self.whichobject == 4:
      glTranslatef(0,0,-1.5)
      gluCylinder(self.quadratic,1,0,3,32,32)
    elif self.whichobject == 5:
      self.discstart += self.inc1
      self.discend += self.inc2
      if self.discstart>=360:
        self.discstart=self.discend=0
        self.inc1,self.inc2 = 0,1
      if self.discend>=360:
        self.inc1,self.inc2 = 1,0
      gluPartialDisk(self.quadratic,.5,1.5,32,32,self.discstart,self.discend-self.discstart)
    self.xrot += self.xspeed
    self.yrot += self.yspeed

screen = Screen()
screen.resize()

running = 1
while running:
  screen.draw()
  pygame.display.flip()
  for e in pygame.event.get():
    if e.type==pygame.VIDEORESIZE:
      screen.wi,screen.hi = e.w,e.h
      screen.resize()
    if e.type==pygame.KEYDOWN:
      if e.key==pygame.K_ESCAPE:
        running = 0
      if e.key==pygame.K_RETURN:
        screen.wi,screen.hi,screen.fullscreen=40,40,0
        screen.resize()
      if e.key==pygame.K_l:
        screen.lighting = not screen.lighting
        if screen.lighting:
          glEnable(GL_LIGHTING)
        else:
          glDisable(GL_LIGHTING)
      if e.key==pygame.K_f:
        screen.filter += 1
        if screen.filter>2:
          screen.filter=0
      if e.key==pygame.K_SPACE:
        screen.whichobject += 1
        if screen.whichobject>5:
          screen.whichobject = 0
  keys = pygame.key.get_pressed()
  if keys[pygame.K_PAGEUP]:
    screen.z += .02
  if keys[pygame.K_PAGEDOWN]:
    screen.z -= .02
  if keys[pygame.K_RIGHT]:
    screen.xspeed += .01
  if keys[pygame.K_LEFT]:
    screen.xspeed -= .01
  if keys[pygame.K_UP]:
    screen.yspeed += .01
  if keys[pygame.K_DOWN]:
    screen.yspeed -= .01
